const expressions = require("./expressions");

function generateCSS(virtualCSS, baseLevel = 3, levelSize = 2) {
    let css = ''
    const addStringToCSS = (string, currentLevel = baseLevel) => {
        css += `${' '.repeat(currentLevel * levelSize)}${string}\n`
    }
    Object.entries(virtualCSS).forEach(([selectorName, properties]) => {
        if (selectorName.includes(',')) {
            selectorName.split(',').forEach((selectorName, index, array) => {
                addStringToCSS(`${selectorName}${index === array.length - 1 ? ' {' : ','}`)
            })
        } else {
            addStringToCSS(`${selectorName} {`)
        }
        Object.entries(properties).forEach(([propertyName, propertyValue]) => {
            addStringToCSS(`${propertyName}: ${propertyValue};`, baseLevel + 1)
        })
        addStringToCSS(`}`)
        css += '\n'
    })
    return css.slice(0, -1)
}

function generateJS(scriptContent, baseLevel = 3, levelSize = 2) {
    return scriptContent
        .split('\n')
        .map(string => `${' '.repeat(baseLevel * levelSize)}${string}`)
        .join(`${' '.repeat(baseLevel * levelSize)}\n`)
}

function generateClassesForSimilarStyles(html) {
    let similarProperties = []
    html.split('>').forEach(string => {
        let newString = string
        let possibleStyle = string.match(expressions.tagStyleList)
        if (possibleStyle !== null) {
            const sortedProperties = `${possibleStyle}`.replace('style="', '').slice(0, -1).split(/;\s*/gms).sort().join('; ')
            similarProperties.push({
                appearedCount: similarProperties.filter(propertiesGroup => propertiesGroup.value === sortedProperties).length + 1,
                value: sortedProperties
            })
        }
    })
    similarProperties = similarProperties.filter(propertiesGroup => propertiesGroup.appearedCount > 1)

    const additionalClasses = {}
    const generateNewAdditionalClass = properties => {
        let possibleSelector = Object.entries(additionalClasses).find(
            ([_, propertiesList]) => (JSON.stringify(properties) === JSON.stringify(propertiesList))
        )
        if (possibleSelector !== undefined) {
            return possibleSelector[0]
        }
        let name = `generated-class-${Object.keys(additionalClasses).length}`
        additionalClasses[`.${name}`] = {...Object.fromEntries(Object.entries(properties).sort((a, b) => {
                if (a[0] < b[0]) {
                    return -1
                }
                if (a[0] > b[0]) {
                    return 1
                }
                return 0
            }))}
        return name
    }

    const newHtml = html
        .split('>')
        .map(string => {
            let newString = string
            let possibleStyle = string.match(expressions.tagStyleList)
            if (possibleStyle !== null) {
                possibleStyle = `${possibleStyle}`.replace('style="', '').slice(0, -1)
                const properties = Object.fromEntries(possibleStyle.split(/;\s*/g).map(property => {
                    let [propertyName, propertyValue] = property.split(/:\s/ms)
                    propertyValue = propertyValue.replace(';', '')
                    return [propertyName, propertyValue]
                }))
                if (similarProperties.find(({value: similarProperty}) => (
                    Object.entries(properties)
                        .map(([propertyName, propertyValue]) => `${propertyName}: ${propertyValue}`)
                        .sort()
                        .join('; ') === similarProperty
                )) === undefined) {
                    return string
                }
                const className = generateNewAdditionalClass(properties)
                newString = string.replace(expressions.tagStyleListAndWhiteSpaces, '')
                let possibleClass = string.match(expressions.tagClassList)
                if (possibleClass !== null) {
                    possibleClass = `${possibleClass}`.replace('class="', '').slice(0, -1)
                    const classList = possibleClass.split(/\s/ms)
                    classList.push(className.replace('.', ''))
                    newString = newString.replace(expressions.tagClassList, `class="${classList.join(' ')}"`)
                } else {
                    newString = `${newString}class="${className}"`
                }
            }
            return newString
        })
        .join('>')
    return {newHtml, additionalClasses}
}

module.exports = {
    generateCSS,
    generateJS,
    generateClassesForSimilarStyles
}
